from io import StringIO
from os.path import dirname, join
from unittest.mock import patch
from py_xlsx_textconv.convert import convert


def test_convert():
    with patch('py_xlsx_textconv.output.stdout',
               new_callable=StringIO) as mock_stdout:
        convert(join(dirname(__file__), '..', 'test.xlsx'))
        assert mock_stdout.getvalue() == '''[Sheet1]\tA1\tB1
[Sheet1]\tA2\t\tC2
[Sheet1]\tA2\t\t\tD3\\nLF
[Sheet2]
'''



from oletools.olevba import VBA_Parser
from py_xlsx_textconv.output import output


def parse_macro(filename: str):
    with open(filename, 'rb') as f:
        content = f.read()
        vbaparser = VBA_Parser(filename, data=content)
        if not vbaparser.detect_vba_macros():
            return

        for filename, _stream_path, vba_filename, vba_code in vbaparser.extract_macros(
        ):
            output(f'[VBA filename]\t{vba_filename}')
            output(vba_code)



from os.path import dirname, join
from unittest.mock import Mock, patch
from py_xlsx_textconv.__main__ import main


def test_main():
    filename = join(dirname(__file__), '..', 'test.xlsx')
    with patch('sys.argv', ['', filename]):
        with patch('py_xlsx_textconv.__main__.convert', Mock()) as mock:
            main()
        mock.assert_called_once_with(filename)


def test_main_no_exists():
    with patch('sys.argv', ['', "not_exists.xlsx"]):
        with patch('py_xlsx_textconv.__main__.convert', Mock()) as mock:
            main()
        mock.assert_not_called()



import sys

# force LF on Windows
stdout = open(sys.__stdout__.fileno(),
              mode=sys.__stdout__.mode,
              buffering=1,
              encoding=sys.__stdout__.encoding,
              errors=sys.__stdout__.errors,
              newline='\n',
              closefd=False)


def output(values: any):
    print(values, file=stdout)



from io import StringIO
from os.path import dirname, join
from unittest.mock import patch

from py_xlsx_textconv.parse_macro import parse_macro


def test_parse_macro():
    with patch('py_xlsx_textconv.output.stdout',
               new_callable=StringIO) as mock_stdout:
        parse_macro(join(dirname(__file__), '..', 'test.xlsm'))
        assert mock_stdout.getvalue() == '''[VBA filename]\tModule1
Attribute VB_Name = "Module1"\r
Option Explicit\r
\r
Sub Macro1()\r
Attribute Macro1.VB_ProcData.VB_Invoke_Func = " \\n14"\r
    Range("A1").Select\r
End Sub\r

[VBA filename]\tThisWorkbook
Attribute VB_Name = "ThisWorkbook"\r
Attribute VB_Base = "0{00020819-0000-0000-C000-000000000046}"\r
Attribute VB_GlobalNameSpace = False\r
Attribute VB_Creatable = False\r
Attribute VB_PredeclaredId = True\r
Attribute VB_Exposed = True\r
Attribute VB_TemplateDerived = False\r
Attribute VB_Customizable = True\r
Option Explicit\r
\r

[VBA filename]\tSheet1
Attribute VB_Name = "Sheet1"\r
Attribute VB_Base = "0{00020820-0000-0000-C000-000000000046}"\r
Attribute VB_GlobalNameSpace = False\r
Attribute VB_Creatable = False\r
Attribute VB_PredeclaredId = True\r
Attribute VB_Exposed = True\r
Attribute VB_TemplateDerived = False\r
Attribute VB_Customizable = True\r
Option Explicit\r
\r

'''
